@@ -21,7 +21,8 @@ using namespace Rcpp;
 using namespace std;
 
 List read_dta(FILE * file, const bool missing) {
-  fseek(file, 18, SEEK_CUR);// stata_dta><header>
+  // stata_dta><header>
+  test("stata_dta><header>", file);
   test("<release>", file);
 
   /*
@@ -73,7 +74,8 @@ List read_dta(FILE * file, const bool missing) {
     break;
   }
 
-  fseek(file, 10, SEEK_CUR); // </release>
+  // </release>
+  test("</release>", file);
   test("<byteorder>", file);
 
   /*
@@ -83,7 +85,8 @@ List read_dta(FILE * file, const bool missing) {
   std::string byteorder(3, '\0');
   readstring(byteorder,file, byteorder.size());
 
-  fseek(file, 12, SEEK_CUR); // </byteorder>
+  // </byteorder>
+  test("</byteorder>", file);
   test("<K>", file);
 
   bool swapit = 0;
@@ -96,7 +99,8 @@ List read_dta(FILE * file, const bool missing) {
   uint16_t k = 0;
   k = readbin(k, file, swapit);
 
-  fseek(file, 4, SEEK_CUR); //</K>
+  //</K>
+  test("</K>", file);
   test("<N>", file);
 
   /*
@@ -112,7 +116,8 @@ List read_dta(FILE * file, const bool missing) {
     n = readbin(n, file, swapit);
   }
 
-  fseek(file, 4, SEEK_CUR); //</N>
+  //</N>
+  test("</N>", file);
   test("<label>", file);
 
   /*
@@ -141,7 +146,8 @@ List read_dta(FILE * file, const bool missing) {
   CharacterVector datalabelCV(1);
   datalabelCV(0) = datalabel;
 
-  fseek(file, 8, SEEK_CUR); //</label>
+  //</label>
+  test("</label>", file);
   test("<timestamp>", file);
 
   /*
@@ -164,7 +170,8 @@ List read_dta(FILE * file, const bool missing) {
   }
 
   CharacterVector timestampCV = timestamp;
-  fseek(file, 21, SEEK_CUR); //</timestamp></header>
+  //</timestamp></header>
+  test("</timestamp></header>", file);
   test("<map>", file);
 
   /*
@@ -194,7 +201,8 @@ List read_dta(FILE * file, const bool missing) {
     map[i] = nmap;
   }
 
-  fseek(file, 6, SEEK_CUR); //</map>
+  //</map>
+  test("</map>", file);
   test("<variable_types>", file);
 
   /*
@@ -216,7 +224,8 @@ List read_dta(FILE * file, const bool missing) {
     vartype[i] = nvartype;
   }
 
-  fseek(file, 17, SEEK_CUR); //</variable_types>
+  //</variable_types>
+  test("</variable_types>", file);
   test("<varnames>", file);
 
   /*
@@ -232,7 +241,8 @@ List read_dta(FILE * file, const bool missing) {
     varnames[i] = nvarnames;
   }
 
-  fseek(file, 11, SEEK_CUR); //</varnames>
+  //</varnames>
+  test("</varnames>", file);
   test("<sortlist>", file);
 
   /*
@@ -252,7 +262,8 @@ List read_dta(FILE * file, const bool missing) {
     sortlist[i] = nsortlist;
   }
 
-  fseek(file, 11, SEEK_CUR); //</sortlist>
+  //</sortlist>
+  test("</sortlist>", file);
   test("<formats>", file);
 
   /*
@@ -269,7 +280,8 @@ List read_dta(FILE * file, const bool missing) {
     formats[i] = nformats;
   }
 
-  fseek(file, 10, SEEK_CUR); //</formats>
+  //</formats>
+  test("</formats>", file);
   test("<value_label_names>",file);
 
   /*
@@ -287,7 +299,8 @@ List read_dta(FILE * file, const bool missing) {
     valLabels[i] = nvalLabels;
   }
 
-  fseek(file, 20, SEEK_CUR); //</value_label_names>
+  //</value_label_names>
+  test("</value_label_names>", file);
   test("<variable_labels>", file);
 
   /*
@@ -303,7 +316,8 @@ List read_dta(FILE * file, const bool missing) {
     varLabels[i] = nvarLabels;
   }
 
-  fseek(file, 18, SEEK_CUR); //</variable_labels>
+  //</variable_labels>
+  test("</variable_labels>", file);
   test("<characteristics>", file);
 
   /*
@@ -325,10 +339,10 @@ List read_dta(FILE * file, const bool missing) {
   List ch = List();
   CharacterVector chs(3);
 
-  while (chtag.compare(tago)==0)
-  {
-    uint32_t nocharacter = 0;
-    nocharacter = readbin(nocharacter, file, swapit);
+    while (chtag.compare(tago)==0)
+    {
+      uint32_t nocharacter = 0;
+      nocharacter = readbin(nocharacter, file, swapit);
 
     std::string chvarname(chlen, '\0');
     std::string chcharact(chlen, '\0');
@@ -347,14 +361,15 @@ List read_dta(FILE * file, const bool missing) {
     // add characteristics to the list
     ch.push_front( chs );
 
-    //fseek(file, 5, SEEK_CUR); // </ch>
+    // </ch>
     test("</ch>", file);
 
     // read next tag
     readstring(tago, file, tago.size());
   }
 
-  fseek(file, 14, SEEK_CUR); //[</ch]aracteristics>
+  //[</ch]aracteristics>
+  test("aracteristics>", file);
   test("<data>", file);
 
   /*
@@ -515,7 +530,8 @@ List read_dta(FILE * file, const bool missing) {
   df.attr("names") = varnames;
   df.attr("class") = "data.frame";
 
-  fseek(file, 7, SEEK_CUR); //</data>
+  //</data>
+  test("</data>", file);
   test("<strls>", file);
 
   /*
@@ -532,12 +548,14 @@ List read_dta(FILE * file, const bool missing) {
   std::string tags(3, '\0');
   readstring(tags, file, tags.size());
 
-  List strlstable = List(); //put strLs into this list
+  //put strLs into a named vector
+  CharacterVector strlvalues(0);
+  CharacterVector strlnames(0);
 
   while(gso.compare(tags)==0)
   {
     CharacterVector strls(2);
-    char erg[22];
+    char ref[22];
 
     // FixMe: Strl in 118
     switch (release)
@@ -549,7 +567,7 @@ List read_dta(FILE * file, const bool missing) {
       v = readbin(v, file, swapit);
       o = readbin(o, file, swapit);
 
-      sprintf(erg, "%010d%010d", v, o);
+      sprintf(ref, "%010d%010d", v, o);
       break;
     }
     case 118:
@@ -561,12 +579,11 @@ List read_dta(FILE * file, const bool missing) {
       o = readbin(o, file, swapit);
       // z = readbin(z, file, swapit);
 
-      sprintf(erg, "%010d%010ld", v, o);
-      // sprintf(erg, "%010ld", z);
+      sprintf(ref, "%010d%010ld", v, o);
+      // sprintf(ref, "%010ld", z);
       break;
     }
     }
-    strls(0) = erg;
 
     // (129 = binary) | (130 = ascii)
     uint8_t t = 0;
@@ -580,15 +597,18 @@ List read_dta(FILE * file, const bool missing) {
     std::string strl(len, '\0');
     readstring(strl, file, strl.size());
 
-    strls(1) = strl;
-
-    strlstable.push_back( strls );
+    strlvalues.push_back( strl );
+    strlnames.push_back( ref );
 
     readstring(tags, file, tags.size());
   }
 
+  // set identifier as name
+  strlvalues.attr("names") = strlnames;
+
   // after strls
-  fseek(file, 5, SEEK_CUR); //[</s]trls>
+  //[</s]trls>
+  test("trls>", file);
   test("<value_labels>", file);
 
   /*
@@ -695,7 +715,8 @@ List read_dta(FILE * file, const bool missing) {
    * close the file
    */
 
-  fseek(file, 10, SEEK_CUR); // [</val]ue_labels>
+  // [</val]ue_labels>
+  test("ue_labels>", file);
   test("</stata_dta>", file);
 
 
@@ -713,7 +734,7 @@ List read_dta(FILE * file, const bool missing) {
   df.attr("version") = versionIV;
   df.attr("label.table") = labelList;
   df.attr("expansion.fields") = ch;
-  df.attr("strl") = strlstable;
+  df.attr("strl") = strlvalues;
   df.attr("byteorder") = wrap(byteorder);
 
   return df;
